@@ -1,5 +1,5 @@
 package Yancy::Model;
-our $VERSION = '1.080';
+our $VERSION = '1.081';
 # ABSTRACT: Model layer for Yancy apps
 
 #pod =head1 SYNOPSIS
@@ -46,6 +46,7 @@ use Mojo::Base -base;
 use Scalar::Util qw( blessed );
 use Mojo::Util qw( camelize );
 use Mojo::Loader qw( load_class );
+use Mojo::Log;
 
 #pod =attr backend
 #pod
@@ -63,6 +64,14 @@ has backend => sub { die "backend is required" };
 
 has namespaces => sub { [qw( Yancy::Model )] };
 
+#pod =attr log
+#pod
+#pod A L<Mojo::Log> object to log messages to.
+#pod
+#pod =cut
+
+has log => sub { Mojo::Log->new };
+
 has _schema => sub { {} };
 
 #pod =method find_class
@@ -174,7 +183,7 @@ Yancy::Model - Model layer for Yancy apps
 
 =head1 VERSION
 
-version 1.080
+version 1.081
 
 =head1 SYNOPSIS
 
@@ -220,6 +229,10 @@ A L<Yancy::Backend> object.
 
 An array of namespaces to find Schema and Item classes. Defaults to C<[ 'Yancy::Model' ]>.
 
+=head2 log
+
+A L<Mojo::Log> object to log messages to.
+
 =head1 METHODS
 
 =head2 find_class
